let data;
let AXIS_INSET;
let AXIS_WIDTH;
let AXIS_HEIGHT;

function preload() {
  data = loadTable('https://gist.githubusercontent.com/mandadi1999/a389d578c2f3a7ff4fa3f9163f5f38c1/raw/6fba8d1e833043c3388951444db8fdc56b6c95c6/gistfile1.txt', 'csv', 'header');
}

function setup() {
  createCanvas(960, 500);
  angleMode(DEGREES);
  rectMode(CENTER);
  AXIS_INSET = 70;
  AXIS_WIDTH = width - 2 * AXIS_INSET;
  AXIS_HEIGHT = height - 2 * AXIS_INSET;
}

function draw() {
  background(250,250,255);

  title("Average Share of Production for Renewables, from 2000 to 2020 for EU Member States", 30);
  drawTicks(AXIS_INSET);
  drawAxis(AXIS_INSET);
  labelAxis(data.columns[0], data.columns[1], 20, 5, 4);
  drawBars(8, AXIS_INSET);
  noLoop();
}

function title(title, inset) {
  textSize(22); fill(37,48,49);
  text(title, inset, inset);
}

function drawAxis(inset) {
  stroke(37,48,49); strokeWeight(3);
  //top left to bottom left
  line(inset, inset, inset, height - inset);
  //bottom left to bottom right
  line(inset, height - inset, width - inset, height - inset);
}

function labelAxis(xLabel, yLabel, inset, numTicksX, numTicksY) {
  textAlign(CENTER, CENTER); textSize(18); noStroke();
  //label x axis
  text(xLabel, width / 2, height - inset);

  //translate and rotate vertically
  translate(inset, height / 2); rotate(-90);
  //label y axis at the origin of translated grid
  text(yLabel, inset, 0);
  resetMatrix();
}

function drawTicks(inset) {
  let fiveScale = getHeight(5);

  for(let i = fiveScale; i < AXIS_HEIGHT + fiveScale; i += fiveScale) {
    stroke(132,130,143); strokeWeight(2);
    line(inset, height - inset - i, inset + AXIS_WIDTH, height - inset - i);
    stroke(37,48,49);
    line(inset - 10, height - inset - i, inset, height - inset - i);
    textSize(15); noStroke(); textAlign(RIGHT);
    text(i / fiveScale * 5.0, inset - 10, height - inset - i + 15);
  }
}

function drawBars(spacing, inset) {
  let runningX = inset + spacing;
  let numRows = data.getRowCount();
  let barWidth = (AXIS_WIDTH - (numRows + 1) * spacing) / numRows;
  runningX += barWidth / 2.0;
  for(let i = 0; i < numRows; i++) {
    let val = data.getColumn(1).at(i);
    let yr = data.getColumn(0).at(i);
    let barHeight = getHeight(val);
    drawBar(runningX, height - inset, barWidth, barHeight, spacing);
    textAlign(CENTER); fill(37,48,49); textSize(15);
    text(yr, runningX, height - inset + 15);
    runningX += spacing + barWidth;

  }
}

function drawBar(x, y, width, height, spacing) {
  noStroke(); fill(88,188,130); translate(0, (height / -2.0) - 1); //subtracting 3 to account for axis width
  rect(x, y, width, height);
  resetMatrix();
}
/*The function sets the fill color to a green shade, translates the canvas so that the rectangle's center aligns with the specified
(x, y) coordinates, draws the rectangle using rect(), and then resets the transformation matrix to its original state using resetMatrix()*/
function getHeight(originalHeight) {
  let maxTick = ceil(max(data.getColumn(1)) / 5.0) * 5.0; //rounded up to nearest five
  return (originalHeight * AXIS_HEIGHT) / maxTick; //from ratio: orig / maxTick = new / axis height
}
